/*
 * Copyright 2013 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Eclipse Public License version 1.0, available at
 * http://www.eclipse.org/legal/epl-v10.html
 */

package org.jboss.forge.addon.configuration;

import java.io.File;

import javax.inject.Inject;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.forge.addon.resource.FileResource;
import org.jboss.forge.addon.resource.ResourceFactory;
import org.jboss.forge.arquillian.AddonDependency;
import org.jboss.forge.arquillian.Dependencies;
import org.jboss.forge.arquillian.archive.ForgeArchive;
import org.jboss.forge.furnace.repositories.AddonDependencyEntry;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(Arquillian.class)
public class ConfigurationFactoryTest
{

   @Deployment
   @Dependencies({
            @AddonDependency(name = "org.jboss.forge.furnace.container:cdi"),
            @AddonDependency(name = "org.jboss.forge.addon:maven"),
            @AddonDependency(name = "org.jboss.forge.addon:projects"),
            @AddonDependency(name = "org.jboss.forge.addon:configuration")
   })
   public static ForgeArchive getDeployment()
   {
      ForgeArchive archive = ShrinkWrap
               .create(ForgeArchive.class)
               .addBeansXML()
               .addAsAddonDependencies(
                        AddonDependencyEntry.create("org.jboss.forge.furnace.container:cdi"),
                        AddonDependencyEntry.create("org.jboss.forge.addon:maven"),
                        AddonDependencyEntry.create("org.jboss.forge.addon:projects"),
                        AddonDependencyEntry.create("org.jboss.forge.addon:configuration")
               );
      return archive;
   }

   @Inject
   private ConfigurationFactory configurationFactory;

   @Inject
   private ResourceFactory resourceFactory;

   @Inject
   private Configuration userConfiguration;

   static
   {
      ConfigurationFactoryImpl.setupTemporaryUserConfig();
   }

   @Test
   public void testUserConfiguration()
   {
      userConfiguration.setProperty("key", "value");
      Assert.assertEquals("value", userConfiguration.getString("key"));
   }

   @Test
   public void testConfigurationFactory() throws Exception
   {
      File file = File.createTempFile("configfactorytest", ".tmp");
      file.delete();
      file.deleteOnExit();
      FileResource<?> resource = resourceFactory.create(file).reify(FileResource.class);
      Assert.assertFalse(resource.exists());
      Configuration config = configurationFactory.getConfiguration(resource);
      config.setProperty("key", "value");
      Assert.assertEquals("value", config.getString("key"));
      // Check if the file was written
      Assert.assertTrue(resource.getSize() > 0L);
   }
}