/**
 * Copyright 2013 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Eclipse Public License version 1.0, available at
 * http://www.eclipse.org/legal/epl-v10.html
 */

package org.jboss.forge.addon.javaee.validation.ui;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.forge.addon.javaee.ProjectHelper;
import org.jboss.forge.addon.parser.java.facets.JavaSourceFacet;
import org.jboss.forge.addon.parser.java.resources.JavaResource;
import org.jboss.forge.addon.projects.Project;
import org.jboss.forge.addon.projects.facets.MetadataFacet;
import org.jboss.forge.addon.ui.controller.CommandController;
import org.jboss.forge.addon.ui.result.Failed;
import org.jboss.forge.addon.ui.result.Result;
import org.jboss.forge.addon.ui.test.UITestHarness;
import org.jboss.forge.arquillian.AddonDependency;
import org.jboss.forge.arquillian.Dependencies;
import org.jboss.forge.arquillian.archive.ForgeArchive;
import org.jboss.forge.furnace.repositories.AddonDependencyEntry;
import org.jboss.forge.parser.java.JavaInterface;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import javax.inject.Inject;

import static org.hamcrest.CoreMatchers.*;

/**
 * Creates a new Bean Validation group
 *
 * @author <a href="antonio.goncalves@gmail.com">Antonio Goncalves</a>
 */
@RunWith(Arquillian.class)
public class NewGroupCommandTest {

  @Deployment
  @Dependencies({
          @AddonDependency(name = "org.jboss.forge.addon:ui"),
          @AddonDependency(name = "org.jboss.forge.addon:ui-test-harness"),
          @AddonDependency(name = "org.jboss.forge.addon:javaee"),
          @AddonDependency(name = "org.jboss.forge.addon:maven")
  })
  public static ForgeArchive getDeployment() {
    return ShrinkWrap
            .create(ForgeArchive.class)
            .addBeansXML()
            .addClass(ProjectHelper.class)
            .addAsAddonDependencies(
                    AddonDependencyEntry.create("org.jboss.forge.furnace.container:cdi"),
                    AddonDependencyEntry.create("org.jboss.forge.addon:projects"),
                    AddonDependencyEntry.create("org.jboss.forge.addon:javaee"),
                    AddonDependencyEntry.create("org.jboss.forge.addon:maven"),
                    AddonDependencyEntry.create("org.jboss.forge.addon:ui"),
                    AddonDependencyEntry.create("org.jboss.forge.addon:ui-test-harness")
            );
  }

  @Inject
  private UITestHarness testHarness;

  @Inject
  private ProjectHelper projectHelper;

  @Test
  public void testCreateNewGroup() throws Exception {
    Project project = projectHelper.createJavaLibraryProject();
    CommandController controller = testHarness.createCommandController(NewGroupCommand.class,
            project.getRootDirectory());
    controller.initialize();
    controller.setValueFor("named", "MyBeanValidationGroup");
    Assert.assertTrue(controller.isValid());
    Assert.assertTrue(controller.canExecute());
    Result result = controller.execute();
    Assert.assertThat(result, is(not(instanceOf(Failed.class))));

    JavaSourceFacet facet = project.getFacet(JavaSourceFacet.class);
    String path = project.getFacet(MetadataFacet.class).getTopLevelPackage() + ".constraints";
    JavaResource javaResource = facet.getJavaResource(path + ".MyBeanValidationGroup");
    Assert.assertNotNull(javaResource);
    Assert.assertThat(javaResource.getJavaSource(), is(instanceOf(JavaInterface.class)));
  }

  @Test
  public void testCreateNewGroupWithTargetPackage() throws Exception {
    Project project = projectHelper.createJavaLibraryProject();
    CommandController controller = testHarness.createCommandController(NewGroupCommand.class,
            project.getRootDirectory());
    controller.initialize();
    controller.setValueFor("named", "MyBeanValidationGroup");
    controller.setValueFor("targetPackage", "org.jboss.forge.test");
    Assert.assertTrue(controller.isValid());
    Assert.assertTrue(controller.canExecute());
    Result result = controller.execute();
    Assert.assertThat(result, is(not(instanceOf(Failed.class))));

    JavaSourceFacet facet = project.getFacet(JavaSourceFacet.class);
    JavaResource javaResource = facet.getJavaResource("org.jboss.forge.test.MyBeanValidationGroup");
    Assert.assertNotNull(javaResource);
    Assert.assertThat(javaResource.getJavaSource(), is(instanceOf(JavaInterface.class)));
  }
}
